;
; ------------------------------------------------------------
;
;  Spartan RPG Engine v0.6 - An RPG Engine Coded in PureBasic
;
;  (c) 2003 - Kenji Gunn
;
; ------------------------------------------------------------
;

; ------------------------------------------------------------
;  Procedure Declarations
; ------------------------------------------------------------
Declare LoadPlayerFrameset(FileName.s, NumOfTiles.w)
Declare LoadTileSet(FileName.s, NumOfTiles.w)

; ------------------------------------------------------------
;  Included Source Files
; ------------------------------------------------------------
IncludeFile "options.pb"

; ------------------------------------------------------------
;  Engine Specifications
; ------------------------------------------------------------
#ScreenResolutionX.w  = 640
#ScreenResolutionY.w  = 480
#ScreenColorDepth.b  = 16
#MapTileSize.b  = 64
#MapTileSizeMinusOne.b  = 63 ;#MapTileSize.b - 1
#SpeedWalk = 2
#SpeedRun = 4

; ------------------------------------------------------------
;  Load Precalculated Tables
; ------------------------------------------------------------
Dim TableMult64.w(10)
OpenFile(0, "tables\mult64.tbl")
For Load.w = 0 To 10
  TableMult64.w(Load.w) = ReadWord()
Next
CloseFile(0)

Dim TableCharMovementFrames.b(3)
OpenFile(0, "tables\mframes.tbl")
For Load.w = 0 To 3
  TableCharMovementFrames.b(Load.w) = ReadByte()
Next
CloseFile(0)

  Dim FramesetX.b(1)
  FramesetX.b(0) = 0
  FramesetX.b(1) = 4  
    Dim FramesetY.b(1)
  FramesetY.b(0) = 8
  FramesetY.b(1) = 12 

;****************************************************************************************
#MapNumOfLayers.b = 2
#MapNumOfTilesX.w = 15
#MapNumOfTilesY.w = 15

#CameraLimitX.l = (#MapTileSize.b  * (#MapNumOfTilesX.w  + 1)) - #ScreenResolutionX.w
#CameraLimitY.l = (#MapTileSize.b  * (#MapNumOfTilesY.w  + 1)) - #ScreenResolutionY.w
#ScreenCenterX.w  = 288
#ScreenCenterY.w  = 208

;  Load map data(Temporary)
Dim MapData.l(#MapNumOfLayers.b, #MapNumOfTilesX.w  + 1, #MapNumOfTilesY.w  + 1)
Restore BaseLayerMapData
For ReadMapDataY  = 0 To 15
  For ReadMapDataX  = 0 To 15
    Read MapData.l(0, ReadMapDataX, ReadMapDataY)
  Next ReadMapDataX
Next ReadMapDataY
;Restore ObstacleLayerMapData:
For ReadMapDataY  = 0 To 15
  For ReadMapDataX  = 0 To 15
    Read MapData.l(1, ReadMapDataX, ReadMapDataY)
  Next ReadMapDataX
Next ReadMapDataY
;Restore ObjectMapData:
For ReadMapDataY  = 0 To 15
  For ReadMapDataX  = 0 To 15
    Read MapData.l(2, ReadMapDataX, ReadMapDataY)
  Next ReadMapDataX
Next ReadMapDataY

HeroPosX  = 128
HeroPosY  = 128

;****************************************************************************************

;  Initialize DirectX 7.0
InitSprite()
InitKeyboard()
InitMouse()
OptionWindow()
SetRefreshRate(60)
SetFrameRate(60)

;  Set priority class to high
SetPriorityClass_(GetCurrentProcess_(),  13)

; Define true and false
#True  = 1
#False  = 0

Dim MapTileSprites(0)
Dim HeroSprites(0)

; ------------------------------------------------------------
;  Load Graphics
; ------------------------------------------------------------
TransparentSpriteColor(-1,  255, 0, 126)
LoadTileSet("images\tileset.png",  12)
LoadPlayerFrameset("images\cyrano.png",  16)

; ------------------------------------------------------------
;  Main Routine
; ------------------------------------------------------------
Repeat
  
  ; ------------------------------------------------------------
  ;  Scan and Process Input
  ; ------------------------------------------------------------
  ExamineKeyboard()

  ArrowDirectionX.b = (KeyboardPushed(#PB_Key_Right) Or 0) - (KeyboardPushed(#PB_Key_Left) Or 0)
  ArrowDirectionY.b = (KeyboardPushed(#PB_Key_Down) Or 0) - (KeyboardPushed(#PB_Key_Up) Or 0)  

  If KeyboardPushed(#PB_Key_LeftShift) <> 0 Or KeyboardPushed(#PB_Key_RightShift) <> 0
    SpeedHero.b = #SpeedRun
  Else
    SpeedHero.b = #SpeedWalk  
  EndIf
  
  ; ------------------------------------------------------------
  ;  Preparation Calculations to Put Tiles
  ; ------------------------------------------------------------
  ;  Convert Hero Posation by pixel, to Posation by tile
  HeroPosByTileX  = HeroPosX  / #MapTileSize.b
  HeroPosByTileY  = HeroPosY  / #MapTileSize.b
  
  ;  Calculate which tile Posation to put from
  TileToPutFromX  = CameraPosX.l  / #MapTileSize.b
  TileToPutFromY  = CameraPosY.l  / #MapTileSize.b
  
  ;  Move Hero in direction of keypress, if no collision is detected
  MovementDirectionX  = ArrowDirectionX.b  * SpeedHero.b
  MovementDirectionY  = ArrowDirectionY.b  * SpeedHero.b
  NewHeroPosByTileX1  = (HeroPosX + MovementDirectionX) / #MapTileSize.b
  NewHeroPosByTileY1  = (HeroPosY + 63) / #MapTileSize.b
  NewHeroPosByTileX2  = (HeroPosX + 63  + MovementDirectionX) / #MapTileSize.b
  NewHeroPosByTileY2  = (HeroPosY + 63) / #MapTileSize.b
  If MapData.l(1, NewHeroPosByTileX1, NewHeroPosByTileY1) = 0 And MapData.l(1, NewHeroPosByTileX2, NewHeroPosByTileY2) = 0
    HeroPosX  = HeroPosX  + MovementDirectionX
  Else
    HeroPosX = HeroPosX
  EndIf
  NewHeroPosByTileX1  = HeroPosX  / #MapTileSize.b
  NewHeroPosByTileY1  = (HeroPosY + 63 + MovementDirectionY) / #MapTileSize.b
  NewHeroPosByTileX2  = (HeroPosX + 63) / #MapTileSize.b
  NewHeroPosByTileY2  = (HeroPosY + 63 + MovementDirectionY) / #MapTileSize.b
  If MapData.l(1, NewHeroPosByTileX1, NewHeroPosByTileY1) = 0 And MapData.l(1, NewHeroPosByTileX2, NewHeroPosByTileY2) = 0
    HeroPosY  = HeroPosY + MovementDirectionY
  Else
    HeroPosY = HeroPosY
  EndIf
  
  ;  Calculate where on screen to start putting tiles
  PosToStartPutTileX  = CameraPosX.l  & #MapTileSizeMinusOne.b
  PosToStartPutTileY  = CameraPosY.l  & #MapTileSizeMinusOne.b
  
  ;  Restrict Camera Within Map
  CameraPosX.l  = HeroPosX  - #ScreenCenterX.w
  CameraPosY.l  = HeroPosY  - #ScreenCenterY.w
  If CameraPosX.l  < 0
    CameraPosX.l  = 0
  EndIf
  If CameraPosY.l  < 0
    CameraPosY.l  = 0
  EndIf
  If CameraPosX.l  > #CameraLimitX.l
    CameraPosX.l  = #CameraLimitX.l
  EndIf
  If CameraPosY.l  > #CameraLimitY.l
    CameraPosY.l  = #CameraLimitY.l
  EndIf
  
  ; ------------------------------------------------------------
  ;  Put Base Layer(Passable Tiles)
  ; ------------------------------------------------------------
  For PutTileY  = 0 To 8
    PreCalPutTileY  = TileToPutFromY  + PutTileY
    For PutTileX  = 0 To 10
      DisplaySprite(MapTileSprites(MapData.l(0,  TileToPutFromX  + PutTileX, PreCalPutTileY)), TableMult64.w(PutTileX) - PosToStartPutTileX, TableMult64.w(PutTileY) - PosToStartPutTileY)
    Next PutTileX
  Next PutTileY
  
  ; ------------------------------------------------------------
  ;  Put Hero Sprite
  ; ------------------------------------------------------------
  FrameX.b = ArrowDirectionX.b + (ArrowDirectionX.b = -1) Or (ArrowDirectionX.b = 1)
  FrameY.b = ArrowDirectionY.b + (ArrowDirectionY.b = -1) Or (ArrowDirectionY.b = 1)

  If ArrowDirectionX.b <> 0 and ArrowDirectionY.b = 0
    Frameset.b = FramesetX.b(FrameX.b)
  EndIf
  If ArrowDirectionY.b <> 0 And ArrowDirectionX.b = 0
    Frameset.b = FramesetY.b(FrameY.b)
  EndIf

  If FrameLifeCycle.b = 7
    FrameLifeCycle.b = 0
    Frame.b  = Frame.b  + 1
    If Frame.b = 4
      Frame.b  = 0
    EndIf
  Else
    FrameLifeCycle.b = FrameLifeCycle.b + 1
  EndIf

  If ArrowDirectionX.b <> 0 Or ArrowDirectionY.b <> 0
      HeroSprite.b = Frameset.b  + TableCharMovementFrames.b(Frame.b)
  EndIf

  DisplayTransparentSprite(HeroSprites(HeroSprite.b),  HeroPosX  - CameraPosX.l, HeroPosY  - CameraPosY.l)
  
  ; ------------------------------------------------------------
  ;  Put Obstacle Layer(Unpassable Tiles)
  ; ------------------------------------------------------------
  For PutTileY  = 0 To 8
    PreCalPutTileY  = TileToPutFromY  + PutTileY
    For PutTileX  = 0 To 10
      If MapData.l(2, TileToPutFromX  + PutTileX, PreCalPutTileY) < >0
        DisplaySprite(MapTileSprites(MapData.l(2, TileToPutFromX  + PutTileX, PreCalPutTileY)), TableMult64.w(PutTileX) - PosToStartPutTileX, TableMult64.w(PutTileY) - PosToStartPutTileY)
      EndIf
    Next PutTileX
  Next PutTileY
  
  If GetTickCount_() => Tick  + 1000
    FrameSec  = Frames
    Frames  = 0
    Tick  = GetTickCount_()
  Else
    Frames  = Frames  + 1
  EndIf
  
  StartDrawing(ScreenOutput())
  DrawingMode(1)
  FrontColor($FF,  $FF, $00)
  Locate(15,  15)
  DrawText("Spartan RPG Engine Ver0.6")
  Locate(15,  31)
  DrawText("Copyright(C) 2003 Kenji Gunn. All Rights Reserved.")
  Locate(15,  47)
  DrawText("FPS: "  + Str(FrameSec))
  StopDrawing()
  
  FlipBuffers()
  
Until KeyboardPushed(#PB_Key_Escape) Or WindowEvent() = #PB_Event_CloseWindow
End

; ------------------------------------------------------------
;  Display Error and Abort Program
; ------------------------------------------------------------
DisplayErrorAndAbort :
CloseScreen()
MessageRequester("Spartan Engine - Error",  ErrorMessage$, #PB_MessageRequester_Ok)
End

; ------------------------------------------------------------
;  Load Sprite from File
; ------------------------------------------------------------
Procedure LoadTileSet(FileName.s, NumOfTiles.w)
  
  Shared NumOfSpritesInMemory
  Dim MapTileSprites(NumOfTiles.w + 100)
  
  NumOfSpritesInMemory  = NumOfSpritesInMemory  + 1
  CreateSprite(NumOfSpritesInMemory,  64, 64, 0)
  MapTileSprites(0)  = NumOfSpritesInMemory
  
  For AllocateSpriteNumbers  = 1 To NumOfTiles.w
    TileNumber  = TileNumber  + 1
    MapTileSprites(TileNumber)  = NumOfSpritesInMemory  + AllocateSpriteNumbers
  Next
  NumOfSpritesInMemory  = NumOfSpritesInMemory  + NumOfTiles.w
  
  LoadSprite(0,  FileName.s, UsePNGImageDecoder())
  
  UseBuffer(0)
  For LoadSprites  = 1 To NumOfTiles.w
    GrabSprite(MapTileSprites(LoadSprites),  GrabSpriteX, GrabSpriteY, 64, 64 , 0)
    GrabSpriteX  = GrabSpriteX  + 64
    If GrabSpriteX  = 640
      GrabSpriteX  = 0
      GrabSpriteY  = GrabSpritesY  + 64
    EndIf
  Next
  UseBuffer(-1)
  FreeSprite(0)
  
EndProcedure

Procedure LoadPlayerFrameset(FileName.s, NumOfTiles.w)
  
  Shared NumOfSpritesInMemory
  Dim HeroSprites(NumOfTiles.w)
  
  For AllocateSpriteNumbers  = 1 To NumOfTiles.w
    HeroSprites(TileNumber)  = NumOfSpritesInMemory  + AllocateSpriteNumbers
    TileNumber  = TileNumber  + 1
  Next
  NumOfSpritesInMemory  = NumOfSpritesInMemory  + NumOfTiles.w
  
  LoadSprite(0, FileName.s, UsePNGImageDecoder())
  
  UseBuffer(0)
  For LoadSprites = 0 To NumOfTiles.w  - 1
    GrabSprite(HeroSprites(LoadSprites),  GrabSpriteX, GrabSpriteY, 64, 75 , 0)
    GrabSpriteX  = GrabSpriteX  + 64
    If GrabSpriteX  = 512
      GrabSpriteX  = 0
      GrabSpriteY  = GrabSpritesY  + 75
    EndIf
  Next
  UseBuffer(-1)
  FreeSprite(0)
EndProcedure

ErrorHandler :
MessageRequester("Spartan RPG Engine",  "An error has occured.", #PB_MessageRequester_Ok)
End

DataSection
BaseLayerMapData :
Data.l 3, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 5
Data.l 6, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8
Data.l 6, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 8
Data.l 6, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 8
Data.l 6, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 8
Data.l 6, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 8
Data.l 6, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 8
Data.l 6, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 8
Data.l 6, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 8
Data.l 6, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 8
Data.l 6, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 8
Data.l 6, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 8
Data.l 6, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 8
Data.l 6, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 8
Data.l 9, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 11
Data.l 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12
ObstacleLayerMapData :
Data.l 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
Data.l 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
Data.l 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1
Data.l 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1
Data.l 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1
Data.l 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1
Data.l 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1
Data.l 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1
Data.l 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1
Data.l 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1
Data.l 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1
Data.l 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1
Data.l 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1
Data.l 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1
Data.l 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1
Data.l 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
ObjectMapData :
Data.l 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
Data.l 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
Data.l 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
Data.l 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
Data.l 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
Data.l 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
Data.l 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
Data.l 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
Data.l 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
Data.l 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
Data.l 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
Data.l 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
Data.l 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
Data.l 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
Data.l 0, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 0
Data.l 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12
EndDataSection
; ExecutableFormat=Windows
; CursorPosition=3
; FirstLine=1
; Executable=D:\}C hLg\}C vWFNg\spartan\spartan.exe
; DisableDebugger
; EOF